@@ -1,5 +1,6 @@
 import unittest
 import os
+import pickle
 
 import numpy as np
 import pandas as pd
@@ -863,6 +864,24 @@ class TestVariogramPlots(unittest.TestCase):
         )
 
 
+class TestVariogramPickling(unittest.TestCase):
+    def setUp(self):
+        # set up default values, whenever c and v are not important
+        np.random.seed(42)
+        self.c = np.random.gamma(10, 4, (150, 2))
+        np.random.seed(42)
+        self.v = np.random.normal(10, 4, 150)
+
+        self.V = Variogram(self.c, self.v, normalize=False)
+
+    def test_save_load_pickle(self):
+        """
+        Only test if loading and saving a pickle works without error
+        """
+        pickle.loads(pickle.dumps(self.V))
+        self.assertTrue()
+
+
 if __name__ == '__main__':  # pragma: no cover
     import os
     os.environ['SKG_SUPRESS'] = 'TRUE'
